@@ -1,5 +1,5 @@
 /*
- * Copyright 2007-2008 by Sascha Hlusiak. <saschahlusiak@freedesktop.org>     
+ * Copyright 2007-2009 by Sascha Hlusiak. <saschahlusiak@freedesktop.org>     
  * Copyright 1995-1999 by Frederic Lepied, France. <Lepied@XFree86.org>       
  *                                                                            
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -26,6 +26,7 @@
 #include "config.h"
 #endif
 
+#include <xorg-server.h>
 #include <stdio.h>
 #include <xorgVersion.h>
 
@@ -33,6 +34,7 @@
 #include <xf86.h>
 #include <xf86Xinput.h>
 #include <exevents.h>		/* Needed for InitValuator/Proximity stuff */
+#include <xf86Opt.h>
 
 #include <math.h>
 #include <xf86Module.h>
@@ -42,6 +44,8 @@
 #include "jstk_axis.h"
 #include "jstk_key.h"
 #include "jstk_options.h"
+#include "jstk_properties.h"
+#include <xserver-properties.h>
 
 #ifdef LINUX_BACKEND
     #include "backend_joystick.h"
@@ -55,43 +59,9 @@
 
 
 #if DEBUG
-    int debug_level = 0;
+    char debug_level = 0;
 #endif
 
-
-/*
- ***************************************************************************
- *
- * jstkConvertProc --
- *
- * Convert valuators to X and Y.
- *
- ***************************************************************************
- */
-
-static Bool
-jstkConvertProc(LocalDevicePtr	local,
-                int             first,
-                int             num,
-                int             v0,
-                int             v1,
-                int             v2,
-                int             v3,
-                int             v4,
-                int             v5,
-                int*            x,
-                int*            y)
-{
-    if (first != 0 || num != 2)
-        return FALSE;
-
-    *x = v0;
-    *y = v1;
-
-    return TRUE;
-}
-
-
 /*
  ***************************************************************************
  *
@@ -105,22 +75,25 @@ jstkConvertProc(LocalDevicePtr	local,
  ***************************************************************************
  */
 static int
-jstkOpenDevice(JoystickDevPtr priv)
+jstkOpenDevice(JoystickDevPtr priv, BOOL probe)
 {
     int fd;
     fd = -1;
 
+    if (probe == FALSE && priv->open_proc)
+        return priv->open_proc(priv, probe);
+
 #ifdef EVDEV_BACKEND
     if (fd == -1)
-        fd = jstkOpenDevice_evdev(priv);
+        fd = jstkOpenDevice_evdev(priv, probe);
 #endif
 #ifdef LINUX_BACKEND
     if (fd == -1)
-        fd = jstkOpenDevice_joystick(priv);
+        fd = jstkOpenDevice_joystick(priv, probe);
 #endif
 #ifdef BSD_BACKEND
     if (fd == -1)
-        fd = jstkOpenDevice_bsd(priv);
+        fd = jstkOpenDevice_bsd(priv, probe);
 #endif
 
     return fd;
@@ -139,21 +112,21 @@ jstkOpenDevice(JoystickDevPtr priv)
  */
 
 static void
-jstkReadProc(LocalDevicePtr local)
+jstkReadProc(InputInfoPtr pInfo)
 {
     JOYSTICKEVENT event;
     int number;
     int i, r;
 
-    JoystickDevPtr priv = local->private;
+    JoystickDevPtr priv = pInfo->private;
 
     do {
         if ((priv->read_proc == NULL) || 
             ((r=priv->read_proc(priv, &event, &number))==0)) {
             xf86Msg(X_WARNING, "JOYSTICK: Read failed. Deactivating device.\n");
 
-            if (local->fd >= 0)
-                RemoveEnabledDevice(local->fd);
+            if (pInfo->fd >= 0)
+                RemoveEnabledDevice(pInfo->fd);
             return;
         }
 
@@ -164,43 +137,43 @@ jstkReadProc(LocalDevicePtr local)
                 priv->button[number].mapping));
 
             switch (priv->button[number].mapping) {
-            case MAPPING_BUTTON:
+            case JSTK_MAPPING_BUTTON:
                 if (priv->mouse_enabled == TRUE) {
-                    xf86PostButtonEvent(local->dev, 0, 
+                    xf86PostButtonEvent(pInfo->dev, 0,
                         priv->button[number].buttonnumber,
                         priv->button[number].pressed, 0, 0);
                 }
                 break;
 
-            case MAPPING_X:
-            case MAPPING_Y:
-            case MAPPING_ZX:
-            case MAPPING_ZY:
+            case JSTK_MAPPING_X:
+            case JSTK_MAPPING_Y:
+            case JSTK_MAPPING_ZX:
+            case JSTK_MAPPING_ZY:
                 if (priv->button[number].pressed == 0) 
                     priv->button[number].currentspeed = 1.0;
                 else if (priv->mouse_enabled == TRUE)
-                    jstkStartButtonAxisTimer(local, number);
+                    jstkStartButtonAxisTimer(pInfo, number);
                 break;
 
-            case MAPPING_KEY:
+            case JSTK_MAPPING_KEY:
                 if (priv->keys_enabled == TRUE)
-                    jstkGenerateKeys(local->dev, 
+                    jstkGenerateKeys(priv->keyboard_device, 
                                      priv->button[number].keys, 
                                      priv->button[number].pressed);
                 break;
 
-            case MAPPING_SPEED_MULTIPLY:
+            case JSTK_MAPPING_SPEED_MULTIPLY:
                 priv->amplify = 1.0;
                 /* Calculate new amplify value by multiplying them all */
                 for (i=0; i<MAXAXES; i++) {
                     if ((priv->button[i].pressed) && 
-                        (priv->button[i].mapping == MAPPING_SPEED_MULTIPLY))
+                        (priv->button[i].mapping == JSTK_MAPPING_SPEED_MULTIPLY))
                         priv->amplify *= priv->button[i].amplify;
                 }
                 DBG(2, ErrorF("Amplify is now %.3f\n", priv->amplify));
                 break;
 
-            case MAPPING_DISABLE:
+            case JSTK_MAPPING_DISABLE:
                 if (priv->button[number].pressed == 1) {
                     if ((priv->mouse_enabled == TRUE) || 
                         (priv->keys_enabled == TRUE))
@@ -215,7 +188,7 @@ jstkReadProc(LocalDevicePtr local)
                     }
                 }
                 break;
-            case MAPPING_DISABLE_MOUSE:
+            case JSTK_MAPPING_DISABLE_MOUSE:
                 if (priv->button[number].pressed == 1) {
                     if (priv->mouse_enabled == TRUE) 
                         priv->mouse_enabled = FALSE;
@@ -224,7 +197,7 @@ jstkReadProc(LocalDevicePtr local)
                         priv->mouse_enabled ? "enabled" : "disabled"));
                 }
                 break;
-            case MAPPING_DISABLE_KEYS:
+            case JSTK_MAPPING_DISABLE_KEYS:
                 if (priv->button[number].pressed == 1) {
                     if (priv->keys_enabled == TRUE) 
                         priv->keys_enabled = FALSE;
@@ -241,7 +214,7 @@ jstkReadProc(LocalDevicePtr local)
 
         /* An axis was moved */
         if ((event == EVENT_AXIS) && 
-            (priv->axis[number].type != TYPE_NONE))
+            (priv->axis[number].type != JSTK_TYPE_NONE))
         {
             DBG(5, ErrorF("Axis %d moved to %d. Type: %d, Mapping: %d\n", 
                           number,
@@ -250,53 +223,42 @@ jstkReadProc(LocalDevicePtr local)
                           priv->axis[number].mapping));
 
             if (priv->axis[number].valuator != -1)
-                xf86PostMotionEvent(local->dev, 1, priv->axis[number].valuator, 
+                xf86PostMotionEvent(pInfo->dev, 1, priv->axis[number].valuator,
                                     1, priv->axis[number].value);
 
             switch (priv->axis[number].mapping) {
-            case MAPPING_X:
-            case MAPPING_Y:
-            case MAPPING_ZX:
-            case MAPPING_ZY:
+            case JSTK_MAPPING_X:
+            case JSTK_MAPPING_Y:
+            case JSTK_MAPPING_ZX:
+            case JSTK_MAPPING_ZY:
                 switch (priv->axis[number].type) {
-                case TYPE_BYVALUE:
-                case TYPE_ACCELERATED:
+                case JSTK_TYPE_BYVALUE:
+                case JSTK_TYPE_ACCELERATED:
                     if (priv->axis[number].value == 0)
                         priv->axis[number].currentspeed = 1.0;
                     if (priv->mouse_enabled == TRUE)
-                        jstkStartAxisTimer(local, number);
+                        jstkStartAxisTimer(pInfo, number);
                     break;
 
-                case TYPE_ABSOLUTE:
+                case JSTK_TYPE_ABSOLUTE:
                     if (priv->mouse_enabled == TRUE)
-                        jstkHandleAbsoluteAxis(local, number);
+                        jstkHandleAbsoluteAxis(pInfo, number);
                     break;
                 default:
                     break;
                 } /* switch (priv->axis[number].type) */
-                break; /* case MAPPING_ZY */
+                break; /* case JSTK_MAPPING_ZY */
 
-            case MAPPING_KEY: if (priv->keys_enabled == TRUE) {
-                if (priv->axis[number].type == TYPE_ACCELERATED) {
-                    if ((priv->axis[number].value > 0) != 
-                        (priv->axis[number].oldvalue > 0))
-                        jstkGenerateKeys(local->dev, 
-                                         priv->axis[number].keys_high,
-                                         (priv->axis[number].value > 0) ? 1:0);
-
-                    if ((priv->axis[number].value < 0) != 
-                        (priv->axis[number].oldvalue < 0))
-                        jstkGenerateKeys(local->dev,
-                                         priv->axis[number].keys_low,
-                                         (priv->axis[number].value < 0) ? 1:0);
-                } else if (priv->axis[number].type == TYPE_BYVALUE) {
-                    if (priv->keys_enabled == TRUE)
-                        jstkStartAxisTimer(local, number);
+            case JSTK_MAPPING_KEY: if (priv->keys_enabled == TRUE) {
+                if (priv->axis[number].type == JSTK_TYPE_ACCELERATED) {
+                    jstkHandlePWMAxis(pInfo, number);
+                } else if (priv->axis[number].type == JSTK_TYPE_BYVALUE) {
+                    jstkStartAxisTimer(pInfo, number);
                 }
                 break;
             }
 
-            case MAPPING_NONE:
+            case JSTK_MAPPING_NONE:
             default:
                 break;
             } /* switch (priv->axis[number].mapping) */
@@ -306,6 +268,15 @@ jstkReadProc(LocalDevicePtr local)
 
 
 
+
+
+static void
+jstkPtrCtrlProc(DeviceIntPtr device, PtrCtrl *ctrl)
+{
+    /* Nothing to do, dix handles all settings */
+}
+
+
 /*
  ***************************************************************************
  *
@@ -321,69 +292,91 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
                       int                what)
 {
     int i;
-    LocalDevicePtr   local = (LocalDevicePtr)pJstk->public.devicePrivate;
-    JoystickDevPtr   priv  = (JoystickDevPtr)XI_PRIVATE(pJstk);
+    InputInfoPtr     pInfo = (InputInfoPtr)pJstk->public.devicePrivate;
+    JoystickDevPtr   priv  = pInfo->private;
+    Atom             btn_labels[BUTTONMAP_SIZE+1] = {0}; /* TODO: fillme */
+    Atom             axes_labels[MAXAXES] = {0}; /* TODO: fillme */
 
     switch (what) {
     case DEVICE_INIT: {
         int m;
+        CARD8 buttonmap[BUTTONMAP_SIZE+1];
         DBG(1, ErrorF("jstkDeviceControlProc what=INIT\n"));
-        if (priv->buttonmap.size != 0) {
-            if (InitButtonClassDeviceStruct(pJstk, priv->buttonmap.size, 
-                priv->buttonmap.map) == FALSE) {
-                ErrorF("unable to allocate Button class device\n");
-                return !Success;
-            }
-            if (InitFocusClassDeviceStruct(pJstk) == FALSE) {
-                ErrorF("unable to init Focus class device\n");
-                return !Success;
-            }
+        /* Probe device and return if error */
+        if (jstkOpenDevice(priv, TRUE) == -1) {
+            return !Success;
+        } else {
+            /* Success. The OpenDevice call already did some initialization
+               like priv->num_buttons, priv->num_axes */
+            priv->close_proc(priv);
         }
-        jstkInitKeys(pJstk, priv);
+
+        for (m=0; m<=BUTTONMAP_SIZE; m++) {
+            buttonmap[m] = m;
+	}
+
+            
+            
+        if (InitButtonClassDeviceStruct(pJstk, BUTTONMAP_SIZE, 
+            btn_labels,
+            buttonmap) == FALSE) {
+            ErrorF("unable to allocate Button class device\n");
+            return !Success;
+        }
+        if (!InitPtrFeedbackClassDeviceStruct(pJstk, jstkPtrCtrlProc))
+            return !Success;
 
         m = 2;
         for (i=0; i<MAXAXES; i++) 
-            if (priv->axis[i].type != TYPE_NONE)
-                priv->axis[i].valuator = m++;
+            if (priv->axis[i].valuator != -1)
+        {
+            DBG(3, ErrorF("Axis %d will be valuator %d\n", i, m));
+            priv->axis[i].valuator = m++;
+        }
 
-        if (InitValuatorClassDeviceStruct(pJstk, 
-                                          m,
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 3
-                                          xf86GetMotionEvents, 
-#endif
-                                          local->history_size,
+        if (InitValuatorClassDeviceStruct(pJstk, m, axes_labels,
+                                          GetMotionHistorySize(),
                                           Relative) == FALSE) {
             ErrorF("unable to allocate Valuator class device\n"); 
             return !Success;
         } else {
             InitValuatorAxisStruct(pJstk,
                                    0, /* valuator num */
+                                   XIGetKnownProperty(AXIS_LABEL_PROP_REL_X),
                                    0, /* min val */
                                    screenInfo.screens[0]->width, /* max val */
                                    1, /* resolution */
                                    0, /* min_res */
-                                   1); /* max_res */
+                                   1, /* max_res */
+                                   Absolute);
             InitValuatorAxisStruct(pJstk,
                                    1, /* valuator num */
+                                   XIGetKnownProperty(AXIS_LABEL_PROP_REL_Y),
                                    0, /* min val */
                                    screenInfo.screens[0]->height, /* max val */
                                    1, /* resolution */
                                    0, /* min_res */
-                                   1); /* max_res */
+                                   1, /* max_res */
+                                   Absolute);
             for (i=0; i<MAXAXES; i++) 
-                if (priv->axis[i].type != TYPE_NONE)
+                if (priv->axis[i].valuator != -1)
             {
                 InitValuatorAxisStruct(pJstk,
                                        priv->axis[i].valuator,
+                                       axes_labels[i],
                                        -32768, /* min val */
                                        32767,  /* max val */
                                        1, /* resolution */
                                        0, /* min_res */
-                                       1); /* max_res */
+                                       1, /* max_res */
+                                       Absolute);
             }
             /* allocate the motion history buffer if needed */
-            xf86MotionHistoryAllocate(local);
+            xf86MotionHistoryAllocate(pInfo);
         }
+
+	jstkInitProperties(pJstk, priv);
+
         break;
     }
 
@@ -391,10 +384,10 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
         DBG(1, ErrorF("jstkDeviceControlProc  what=ON name=%s\n", 
                       priv->device));
 
-        if (jstkOpenDevice(priv) != -1) {
+        if (jstkOpenDevice(priv, FALSE) != -1) {
             pJstk->public.on = TRUE;
-            local->fd = priv->fd;
-            AddEnabledDevice(local->fd);
+            pInfo->fd = priv->fd;
+            AddEnabledDevice(pInfo->fd);
         } else return !Success;
         break;
 
@@ -409,10 +402,16 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
             priv->timerrunning = FALSE;
             TimerCancel(priv->timer);
         }
+        for (i = 0; i < MAXAXES; i++)
+            if (priv->axis[i].timerrunning)
+        {
+            priv->axis[i].timerrunning = FALSE;
+            TimerCancel(priv->axis[i].timer);
+        }
 
-        if (local->fd >= 0)
-            RemoveEnabledDevice(local->fd);
-        local->fd = -1;
+        if (pInfo->fd >= 0)
+            RemoveEnabledDevice(pInfo->fd);
+        pInfo->fd = -1;
         if (priv->close_proc)
             priv->close_proc(priv);
         pJstk->public.on = FALSE;
@@ -426,8 +425,6 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
     return Success;
 }
 
-
-
 /*
  ***************************************************************************
  *
@@ -435,44 +432,46 @@ jstkDeviceControlProc(DeviceIntPtr       pJstk,
  *
  * Called when a device will be instantiated
  *
+ * This is a tad complicated. NewInputDeviceRequest(), which we use to
+ * hotplug a keyboard device,. enables the device, so we need to make sure
+ * that all options for the dependent device are set correctly.
+ *
+ * This means that we parse the keyboard-specific options into the
+ * keyboard device's PreInit, and re-use the keyboard's priv field.
+ *
  ***************************************************************************
  */
 
-static InputInfoPtr
-jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
+static int
+jstkCorePreInit(InputDriverPtr drv, InputInfoPtr pInfo, int flags)
 {
-    LocalDevicePtr      local = NULL;
+    InputInfoPtr        keyboard_device;
     JoystickDevPtr      priv = NULL;
     char                *s;
     int                 i, j;
 
-    local = xf86AllocateInput(drv, 0);
-    if (!local) {
-        goto SetupProc_fail;
-    }
+    s = xf86CheckStrOption(pInfo->options, "_source", "");
+    if (strcmp(s, "_driver/joystick") == 0)
+        return jstkKeyboardPreInit(drv, pInfo, flags);
 
-    local->private = (JoystickDevPtr)xalloc(sizeof(JoystickDevRec));
-    priv = (JoystickDevPtr)local->private;
+    pInfo->device_control = jstkDeviceControlProc;
+    pInfo->read_input = jstkReadProc;
+    pInfo->control_proc = NULL;
+    pInfo->switch_mode = NULL;
+    pInfo->fd = -1;
+    pInfo->dev = NULL;
+    pInfo->type_name = XI_JOYSTICK;
 
-    local->name   = dev->identifier;
-    local->flags  = XI86_POINTER_CAPABLE;
-    local->flags |= XI86_KEYBOARD_CAPABLE;
-    local->flags |= XI86_SEND_DRAG_EVENTS;
-    local->device_control = jstkDeviceControlProc;
-    local->read_input = jstkReadProc;
-    local->close_proc = NULL;
-    local->control_proc = NULL;
-    local->switch_mode = NULL;
-    local->conversion_proc = jstkConvertProc;
-    local->fd = -1;
-    local->dev = NULL;
-    local->private = priv;
-    local->type_name = XI_MOUSE;
-    local->history_size = 0;
-    local->always_core_feedback = 0;
-    local->conf_idev = dev;
+    keyboard_device = jstkKeyboardHotplug(pInfo, flags);
+    if (!keyboard_device)
+        return BadAlloc;
+
+    pInfo->private = priv = keyboard_device->private;
 
     priv->fd = -1;
+    priv->open_proc = NULL;
+    priv->read_proc = NULL;
+    priv->close_proc = NULL;
     priv->device = NULL;
     priv->devicedata = NULL;
     priv->timer = NULL;
@@ -480,125 +479,97 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     priv->mouse_enabled = TRUE;
     priv->keys_enabled = TRUE;
     priv->amplify = 1.0f;
-    priv->buttonmap.size = 0;
-    priv->keymap.size = 1;
-    memset(priv->keymap.map, NoSymbol, sizeof(priv->keymap.map));
-    priv->repeat_delay = 0;
-    priv->repeat_interval = 0;
+    priv->keyboard_device = keyboard_device;
+    priv->num_axes    = MAXAXES;
+    priv->num_buttons = MAXBUTTONS;
 
     /* Initialize default mappings */
     for (i=0; i<MAXAXES; i++) {
         priv->axis[i].value        = 0;
         priv->axis[i].oldvalue     = 0;
         priv->axis[i].deadzone     = 5000;
-        priv->axis[i].type         = TYPE_NONE;
-        priv->axis[i].mapping      = MAPPING_NONE;
+        priv->axis[i].type         = JSTK_TYPE_NONE;
+        priv->axis[i].mapping      = JSTK_MAPPING_NONE;
         priv->axis[i].currentspeed = 0.0f;
         priv->axis[i].amplify      = 1.0f;
         priv->axis[i].valuator     = -1;
         priv->axis[i].subpixel     = 0.0f;
+        priv->axis[i].timer        = NULL;
+        priv->axis[i].timerrunning = FALSE;
+        priv->axis[i].key_isdown   = 0;
         for (j=0; j<MAXKEYSPERBUTTON; j++)
             priv->axis[i].keys_low[j] = priv->axis[i].keys_high[j] = 0;
     }
     for (i=0; i<MAXBUTTONS; i++) {
         priv->button[i].pressed      = 0;
         priv->button[i].buttonnumber = 0;
-        priv->button[i].mapping      = MAPPING_NONE;
+        priv->button[i].mapping      = JSTK_MAPPING_NONE;
         priv->button[i].currentspeed = 1.0f;
         priv->button[i].subpixel     = 0.0f;
+        priv->button[i].amplify      = 1.0;
         for (j=0; j<MAXKEYSPERBUTTON; j++)
             priv->button[i].keys[j] = 0;
     }
 
-    priv->buttonmap.map[0] = 0;
-
     /* First three joystick buttons generate mouse clicks */
-    priv->button[0].mapping      = MAPPING_BUTTON;
-    priv->button[0].buttonnumber = jstkGetButtonNumberInMap(priv, 1);
-    priv->button[1].mapping      = MAPPING_BUTTON;
-    priv->button[1].buttonnumber = jstkGetButtonNumberInMap(priv, 2);
-    priv->button[2].mapping      = MAPPING_BUTTON;
-    priv->button[2].buttonnumber = jstkGetButtonNumberInMap(priv, 3);
+    priv->button[0].mapping      = JSTK_MAPPING_BUTTON;
+    priv->button[0].buttonnumber = 1;
+    priv->button[1].mapping      = JSTK_MAPPING_BUTTON;
+    priv->button[1].buttonnumber = 2;
+    priv->button[2].mapping      = JSTK_MAPPING_BUTTON;
+    priv->button[2].buttonnumber = 3;
 
     /* First two axes are a stick for moving */
-    priv->axis[0].type      = TYPE_BYVALUE;
-    priv->axis[0].mapping   = MAPPING_X;
-    priv->axis[1].type      = TYPE_BYVALUE;
-    priv->axis[1].mapping   = MAPPING_Y;
+    priv->axis[0].type      = JSTK_TYPE_BYVALUE;
+    priv->axis[0].mapping   = JSTK_MAPPING_X;
+    priv->axis[1].type      = JSTK_TYPE_BYVALUE;
+    priv->axis[1].mapping   = JSTK_MAPPING_Y;
 
     /* Next two axes are a stick for scrolling */
-    priv->axis[2].type      = TYPE_BYVALUE;
-    priv->axis[2].mapping   = MAPPING_ZX;
-    priv->axis[3].type      = TYPE_BYVALUE;
-    priv->axis[3].mapping   = MAPPING_ZY;
+    priv->axis[2].type      = JSTK_TYPE_BYVALUE;
+    priv->axis[2].mapping   = JSTK_MAPPING_ZX;
+    priv->axis[3].type      = JSTK_TYPE_BYVALUE;
+    priv->axis[3].mapping   = JSTK_MAPPING_ZY;
 
     /* Next two axes are a pad for moving */
-    priv->axis[4].type      = TYPE_ACCELERATED;
-    priv->axis[4].mapping   = MAPPING_X;
-    priv->axis[5].type      = TYPE_ACCELERATED;
-    priv->axis[5].mapping   = MAPPING_Y;
-
-    priv->buttonmap.scrollbutton[0] = jstkGetButtonNumberInMap(priv, 4);
-    priv->buttonmap.scrollbutton[1] = jstkGetButtonNumberInMap(priv, 5);
-    priv->buttonmap.scrollbutton[2] = jstkGetButtonNumberInMap(priv, 6);
-    priv->buttonmap.scrollbutton[3] = jstkGetButtonNumberInMap(priv, 7);
-
-
-    xf86CollectInputOptions(local, NULL, NULL);
-    xf86OptionListReport(local->options);
+    priv->axis[4].type      = JSTK_TYPE_ACCELERATED;
+    priv->axis[4].mapping   = JSTK_MAPPING_X;
+    priv->axis[5].type      = JSTK_TYPE_ACCELERATED;
+    priv->axis[5].mapping   = JSTK_MAPPING_Y;
 
     /* Joystick device is mandatory */
-    priv->device = xf86SetStrOption(dev->commonOptions, "Device", NULL);
+    priv->device = xf86SetStrOption(pInfo->options, "Device", NULL);
     if (!priv->device)
-        priv->device = xf86SetStrOption(dev->commonOptions, "Path", NULL);
+        priv->device = xf86SetStrOption(pInfo->options, "Path", NULL);
 
     if (!priv->device) {
-        xf86Msg (X_ERROR, "%s: No Device specified.\n", local->name);
+        xf86Msg (X_ERROR, "%s: No Device specified.\n", pInfo->name);
         goto SetupProc_fail;
     }
 
-    xf86ProcessCommonOptions(local, local->options);
-
 #if DEBUG
-    debug_level = xf86SetIntOption(dev->commonOptions, "DebugLevel", 0);
+    debug_level = xf86SetIntOption(pInfo->options, "DebugLevel", 0);
     if (debug_level > 0) {
         xf86Msg(X_CONFIG, "%s: debug level set to %d\n", 
-                local->name, debug_level);
+                pInfo->name, debug_level);
     }
 #else
-    if (xf86SetIntOption(dev->commonOptions, "DebugLevel", 0) != 0) {
+    if (xf86SetIntOption(pInfo->options, "DebugLevel", 0) != 0) {
         xf86Msg(X_WARNING, "%s: DebugLevel: Compiled without debugging support!\n", 
-                local->name);
+                pInfo->name);
     }
 #endif
 
-    /* Parse option for autorepeat */
-    if ((s = xf86SetStrOption(dev->commonOptions, "AutoRepeat", NULL))) {
-        int delay, rate;
-        if (sscanf(s, "%d %d", &delay, &rate) != 2) {
-            xf86Msg(X_ERROR, "%s: \"%s\" is not a valid AutoRepeat value", 
-                    local->name, s);
-        } else {
-            priv->repeat_delay = delay;
-            if (rate != 0)
-                priv->repeat_interval = 1000/rate;
-            else priv->repeat_interval = 0;
-            DBG(1, xf86Msg(X_CONFIG, "Autorepeat set to delay=%d, interval=%d\n",
-                           priv->repeat_delay,priv->repeat_interval));
-        }
-        xfree(s);
-    }
-
-    priv->mouse_enabled = xf86SetBoolOption(dev->commonOptions, "StartMouseEnabled", TRUE);
-    priv->keys_enabled = xf86SetBoolOption(dev->commonOptions, "StartKeysEnabled", TRUE);
+    priv->mouse_enabled = xf86SetBoolOption(pInfo->options, "StartMouseEnabled", TRUE);
+    priv->keys_enabled = xf86SetBoolOption(pInfo->options, "StartKeysEnabled", TRUE);
 
     /* Process button mapping options */
     for (i=0; i<MAXBUTTONS; i++) {
         char p[64];
         sprintf(p,"MapButton%d",i+1);
-        s = xf86SetStrOption(dev->commonOptions, p, NULL);
+        s = xf86SetStrOption(pInfo->options, p, NULL);
         if (s != NULL) {
-            jstkParseButtonOption(s, priv, i, local->name);
+            jstkParseButtonOption(s, priv, i, pInfo->name);
         }
         DBG(1, xf86Msg(X_CONFIG, "Button %d mapped to %d\n", i+1, 
                        priv->button[i].mapping));
@@ -608,9 +579,9 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     for (i=0; i<MAXAXES; i++) {
         char p[64];
         sprintf(p,"MapAxis%d",i+1);
-        s = xf86SetStrOption(dev->commonOptions, p, NULL);
+        s = xf86SetStrOption(pInfo->options, p, NULL);
         if (s != NULL) {
-            jstkParseAxisOption(s, priv, &priv->axis[i], local->name);
+            jstkParseAxisOption(s, priv, &priv->axis[i], pInfo->name);
         }
         DBG(1, xf86Msg(X_CONFIG, 
                        "Axis %d type is %d, mapped to %d, amplify=%.3f\n", i+1, 
@@ -619,18 +590,17 @@ jstkCorePreInit(InputDriverPtr drv, IDevPtr dev, int flags)
                        priv->axis[i].amplify));
     }
 
-    /* return the LocalDevice */
-    local->flags |= XI86_CONFIGURED;
-
-    return (local);
+    return Success;
 
 SetupProc_fail:
-    if (priv)
-        xfree(priv);
-    if (local)
-        local->private = NULL;
-    return NULL;
-/*    return (local); */ /* Makes X segfault on error */
+    if (priv) {
+        free(priv);
+        if (keyboard_device)
+            keyboard_device->private = NULL;
+    }
+    if (pInfo)
+        pInfo->private = NULL;
+    return BadValue;
 }
 
 
@@ -647,16 +617,20 @@ SetupProc_fail:
 
 static void
 jstkCoreUnInit(InputDriverPtr    drv,
-               LocalDevicePtr    local,
+               InputInfoPtr      pInfo,
                int               flags)
 {
-    JoystickDevPtr device = (JoystickDevPtr) local->private;
+    JoystickDevPtr device = (JoystickDevPtr) pInfo->private;
 
-    jstkDeviceControlProc(local->dev, DEVICE_OFF);
+    if (device->keyboard_device != NULL)
+    {
+        xf86DisableDevice(device->keyboard_device->dev, TRUE);
+        device->keyboard_device = NULL;
+    }
 
-    xfree (device);
-    local->private = NULL;
-    xf86DeleteInput(local, 0);
+    free (device);
+    pInfo->private = NULL;
+    xf86DeleteInput(pInfo, 0);
 }
 
 
@@ -668,11 +642,9 @@ _X_EXPORT InputDriverRec JOYSTICK = {
     NULL,
     jstkCorePreInit,
     jstkCoreUnInit,
-    NULL,
-    0
+    NULL
 };
 
-
 /*
  ***************************************************************************
  *
